from typing import Optional, Dict, Any

import aioredis
import ujson


class RedisConnection:
    redis: Optional[aioredis.Redis] = None
    connection_info: Dict[str, Any]

    @classmethod
    async def _get_redis_connection(cls):
        if cls.redis and not cls.redis.closed:
            return cls.redis

        cls.redis = await aioredis.create_redis_pool(cls.connection_info)
        return cls.redis

    @classmethod
    async def initialize(cls, connection_info):
        cls.connection_info = connection_info
        await cls._get_redis_connection()

    @classmethod
    async def release(cls):
        if cls.redis:
            cls.redis.close()
            await cls.redis.wait_closed()

        cls.redis = None

    @classmethod
    async def set(
        cls, key: str, value: Any, **kwargs
    ) -> None:
        redis = await cls._get_redis_connection()

        dumped_value = ujson.dumps(value)
        await redis.set(key, dumped_value, **kwargs)

    @classmethod
    async def get(cls, key: str) -> Any:
        redis = await cls._get_redis_connection()
        value = await redis.get(key)
        value = ujson.loads(value) if value else None

        return value

    @classmethod
    async def delete(cls, *keys: str):
        redis = await cls._get_redis_connection()
        await redis.delete(*keys)



import warnings
from abc import ABC, abstractmethod
from datetime import datetime
from typing import Dict, Any

import aioredis

from sanic_jwt_extended.redis import RedisConnection


class BlacklistABC(ABC):
    @abstractmethod
    async def register(self, token):
        pass

    @abstractmethod
    async def is_blacklisted(self, token):
        pass


class InMemoryBlacklist(BlacklistABC):
    def __init__(self):
        self.blacklist = []
        warnings.warn(
            "Using in-memory blacklist is not recommended for production environment"
        )

    async def register(self, token):
        self.blacklist.append(token.jti)

    async def is_blacklisted(self, token):
        return token.jti in self.blacklist


class RedisBlacklist(BlacklistABC):

    def __init__(self, connection_info):
        self.connection_info = connection_info

    async def register(self, token):
        if not RedisConnection.redis:
            await RedisConnection.initialize(self.connection_info)

        kwargs = {}

        if token.exp:
            kwargs["expire"] = token.exp

        await RedisConnection.set(token.jti.hex, token.raw_jwt, **kwargs)

    async def is_blacklisted(self, token):
        if not RedisConnection.redis:
            await RedisConnection.initialize(self.connection_info)

        return bool(RedisConnection.get(token.jti.hex))
